@@ -18,6 +18,7 @@ package com.google.i18n.phonenumbers;
 
 import com.google.i18n.phonenumbers.Phonemetadata.NumberFormat;
 import com.google.i18n.phonenumbers.Phonemetadata.PhoneMetadata;
+import com.google.i18n.phonenumbers.internal.RegexCache;
 
 import java.util.ArrayList;
 import java.util.Iterator;
@@ -82,8 +83,8 @@ public class AsYouTypeFormatter {
   // prevents invalid punctuation (such as the star sign in Israeli star numbers) getting into the
   // output of the AYTF.
   private static final Pattern ELIGIBLE_FORMAT_PATTERN =
-      Pattern.compile("[" + PhoneNumberUtil.VALID_PUNCTUATION + "]*" +
-          "(\\$\\d" + "[" + PhoneNumberUtil.VALID_PUNCTUATION + "]*)+");
+      Pattern.compile("[" + PhoneNumberUtil.VALID_PUNCTUATION + "]*"
+          + "(\\$\\d" + "[" + PhoneNumberUtil.VALID_PUNCTUATION + "]*)+");
   // A set of characters that, if found in a national prefix formatting rules, are an indicator to
   // us that we should separate the national prefix from the number when formatting.
   private static final Pattern NATIONAL_PREFIX_SEPARATORS_PATTERN = Pattern.compile("[- ]");
@@ -115,7 +116,7 @@ public class AsYouTypeFormatter {
   private StringBuilder nationalNumber = new StringBuilder();
   private List<NumberFormat> possibleFormats = new ArrayList<NumberFormat>();
 
-    // A cache for frequently used country-specific regular expressions.
+  // A cache for frequently used country-specific regular expressions.
   private RegexCache regexCache = new RegexCache(64);
 
   /**
@@ -179,9 +180,10 @@ public class AsYouTypeFormatter {
         : currentMetadata.numberFormats();
     boolean nationalPrefixIsUsedByCountry = currentMetadata.hasNationalPrefix();
     for (NumberFormat format : formatList) {
-      if (!nationalPrefixIsUsedByCountry || isCompleteNumber ||
-          format.isNationalPrefixOptionalWhenFormatting() ||
-          PhoneNumberUtil.formattingRuleHasFirstGroupOnly(
+      if (!nationalPrefixIsUsedByCountry
+          || isCompleteNumber
+          || format.getNationalPrefixOptionalWhenFormatting()
+          || PhoneNumberUtil.formattingRuleHasFirstGroupOnly(
               format.getNationalPrefixFormattingRule())) {
         if (isFormatEligible(format.getFormat())) {
           possibleFormats.add(format);
@@ -358,6 +360,7 @@ public class AsYouTypeFormatter {
           extractedNationalPrefix = removeNationalPrefixFromNationalNumber();
           return attemptToChooseFormattingPattern();
         }
+        // fall through
       default:
         if (isExpectingCountryCallingCode) {
           if (attemptToExtractCountryCallingCode()) {
@@ -390,6 +393,9 @@ public class AsYouTypeFormatter {
     ableToFormat = true;
     isExpectingCountryCallingCode = false;
     possibleFormats.clear();
+    lastMatchPosition = 0;
+    formattingTemplate.setLength(0);
+    currentFormattingPattern = "";
     return attemptToChooseFormattingPattern();
   }
 
@@ -414,13 +420,13 @@ public class AsYouTypeFormatter {
   }
 
   private boolean isDigitOrLeadingPlusSign(char nextChar) {
-    return Character.isDigit(nextChar) ||
-        (accruedInput.length() == 1 &&
-         PhoneNumberUtil.PLUS_CHARS_PATTERN.matcher(Character.toString(nextChar)).matches());
+    return Character.isDigit(nextChar)
+        || (accruedInput.length() == 1
+            && PhoneNumberUtil.PLUS_CHARS_PATTERN.matcher(Character.toString(nextChar)).matches());
   }
 
   /**
-   * Check to see if there is an exact pattern match for these digits. If so, we should use this
+   * Checks to see if there is an exact pattern match for these digits. If so, we should use this
    * instead of any other formatting template whose leadingDigitsPattern also matches the input.
    */
   String attemptToFormatAccruedDigits() {
@@ -445,10 +451,11 @@ public class AsYouTypeFormatter {
     if (!ableToFormat) {
       return originalPosition;
     }
-    int accruedInputIndex = 0, currentOutputIndex = 0;
+    int accruedInputIndex = 0;
+    int currentOutputIndex = 0;
     while (accruedInputIndex < positionToRemember && currentOutputIndex < currentOutput.length()) {
-      if (accruedInputWithoutFormatting.charAt(accruedInputIndex) ==
-          currentOutput.charAt(currentOutputIndex)) {
+      if (accruedInputWithoutFormatting.charAt(accruedInputIndex)
+          == currentOutput.charAt(currentOutputIndex)) {
         accruedInputIndex++;
       }
       currentOutputIndex++;
@@ -463,8 +470,8 @@ public class AsYouTypeFormatter {
    */
   private String appendNationalNumber(String nationalNumber) {
     int prefixBeforeNationalNumberLength = prefixBeforeNationalNumber.length();
-    if (shouldAddSpaceAfterNationalPrefix && prefixBeforeNationalNumberLength > 0 &&
-        prefixBeforeNationalNumber.charAt(prefixBeforeNationalNumberLength - 1)
+    if (shouldAddSpaceAfterNationalPrefix && prefixBeforeNationalNumberLength > 0
+        && prefixBeforeNationalNumber.charAt(prefixBeforeNationalNumberLength - 1)
             != SEPARATOR_BEFORE_NATIONAL_NUMBER) {
       // We want to add a space after the national prefix if the national prefix formatting rule
       // indicates that this would normally be done, with the exception of the case where we already
@@ -523,8 +530,8 @@ public class AsYouTypeFormatter {
     // that national significant numbers in NANPA always start with [2-9] after the national prefix.
     // Numbers beginning with 1[01] can only be short/emergency numbers, which don't need the
     // national prefix.
-    return (currentMetadata.getCountryCode() == 1) && (nationalNumber.charAt(0) == '1') &&
-           (nationalNumber.charAt(1) != '0') && (nationalNumber.charAt(1) != '1');
+    return (currentMetadata.getCountryCode() == 1) && (nationalNumber.charAt(0) == '1')
+        && (nationalNumber.charAt(1) != '0') && (nationalNumber.charAt(1) != '1');
   }
 
   // Returns the national prefix extracted, or an empty string if it is not present.
@@ -563,8 +570,8 @@ public class AsYouTypeFormatter {
    */
   private boolean attemptToExtractIdd() {
     Pattern internationalPrefix =
-        regexCache.getPatternForRegex("\\" + PhoneNumberUtil.PLUS_SIGN + "|" +
-            currentMetadata.getInternationalPrefix());
+        regexCache.getPatternForRegex("\\" + PhoneNumberUtil.PLUS_SIGN + "|"
+            + currentMetadata.getInternationalPrefix());
     Matcher iddMatcher = internationalPrefix.matcher(accruedInputWithoutFormatting);
     if (iddMatcher.lookingAt()) {
       isCompleteNumber = true;
@@ -637,6 +644,8 @@ public class AsYouTypeFormatter {
   }
 
   private String inputDigitHelper(char nextChar) {
+    // Note that formattingTemplate is not guaranteed to have a value, it could be empty, e.g.
+    // when the next digit is entered after extracting an IDD or NDD.
     Matcher digitMatcher = DIGIT_PATTERN.matcher(formattingTemplate);
     if (digitMatcher.find(lastMatchPosition)) {
       String tempTemplate = digitMatcher.replaceFirst(Character.toString(nextChar));
